BECH32_ADDR_ACC_PREFIX = "band"
BECH32_ADDR_VAL_PREFIX = "bandvaloper"
BECH32_ADDR_CONS_PREFIX = "bandvalcons"

BECH32_PUBKEY_ACC_PREFIX = "bandpub"
BECH32_PUBKEY_VAL_PREFIX = "bandvaloperpub"
BECH32_PUBKEY_CONS_PREFIX = "bandvalconspub"

DEFAULT_DERIVATION_PATH = "m/44'/494'/0'/0/0"
DEFAULT_LEDGER_DERIVATION_PATH = "m/44'/118'/0'/0/0"
